/***************************************************************************
                          Session.h  -  description
                             -------------------
    begin                : Fri May 4 2002
    copyright            : (C) 2002 by Henrik Baastrup
    email                : hba@libero.it
    home page            : http://digilander.iol.it/hba/Linux/index.html
 ***************************************************************************/

/***************************************************************************
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU LGPL (Lesser general Public License) 
 *   as published by the Free Software Foundation:
 *   http://www.gnu.org/copyleft/lesser.html or in the file copying.txt
 *
 ***************************************************************************/
#ifndef _SESSION_H_
#define _SESSION_H_

#include <time.h>
#ifdef LINUX
 #include <stdlib.h>
 #include <stdio.h>
 #include <string.h>
 #include <unistd.h>
 #include <sys/socket.h>
 #include <pthread.h>
#elif defined(WIN32)
 #include <stdio.h>
 #include <string.h>
 #include <winsock.h>
#endif

#ifndef SOCKET
 #define SOCKET int
#endif

class MiniHTTP;
class HttpService;

class HttpSession
{
	private:
		time_t m_touchTime;
		MiniHTTP *m_miniHTTP;
		HttpSession *m_nextSession;
		SOCKET m_clientSocketFd;
		struct sockaddr_in *m_clientAddr;
#ifdef LINUX
		pthread_t  m_threadId;
		pthread_attr_t m_threadAttr;
#elif defined(WIN32)
		HANDLE m_threadId;
#endif
		char m_serverName[256];
		char m_httpMethod[10];
		char m_requestURI[1024];
		char m_httpRequestVersion[15];
		char m_httpSupportedVersion[15];
		char m_contentType[256];
		long m_contentLength;
		long m_connectionKeepAlive;

		char *parseHttpHeader(char *);

	public:
		HttpSession(MiniHTTP *, struct sockaddr_in *, int);
		~HttpSession();

		void setNextSession(HttpSession *s) {m_nextSession = s;};
		HttpSession *getNextSession() {return m_nextSession;};
		SOCKET getClientSocket() {return m_clientSocketFd;};
		void runSession(SOCKET);
		void goSession();

		int sendStr(char *);
		void writeOkHeader();
		void writeError(char *headerStr, char *bodyStr);

#ifdef LINUX
		pthread_t getThreadId() {return m_threadId;};
#elif defined(WIN32)
		HANDLE getThreadId() {return m_threadId;};
#endif
		void nullSessionThreadId() {m_threadId = 0;};
		char *getHomeDir();
		char *getRequstURI() {return m_requestURI;};
		char *getContentType() {return m_contentType;};
		void setContentType(char *s) {strncpy(m_contentType, s, 256); m_contentType[255] = '\0';};
		long getContentLength() {return m_contentLength;};
		void setContentLength(long length) {m_contentLength = length;};
		long isConnectionKeepAlive() {return m_connectionKeepAlive;};
		char *getHttpSupportedVersion() {return m_httpSupportedVersion;};
		void callGetFunction(char *msg);
		int callPostFunction(char *msg, char **rsp);
		time_t getTouchTime() {return m_touchTime;};
}; //HttpSession

#endif
